@@ -81,20 +81,28 @@ get_default (BonoboConfigDatabase   *cd,
 	DataBaseInfo *info;
 	GList *l;
 
+	bonobo_object_ref (BONOBO_OBJECT (cd));
+
 	for (l = cd->priv->db_list; l != NULL; l = l->next) {
 		info = (DataBaseInfo *)l->data;
 
 		value = Bonobo_ConfigDatabase_getValue (info->db, key, locale,
 							ev);
-		if (BONOBO_EX (ev))
+		if (BONOBO_EX (ev)) {
+			bonobo_object_unref (BONOBO_OBJECT (cd));
 			return NULL;
+		}
 
-		if (value)
+		if (value) {
+			bonobo_object_unref (BONOBO_OBJECT (cd));
 			return value;
-
+		}
 	}
 
+	bonobo_object_unref (BONOBO_OBJECT (cd));
+
 	bonobo_exception_set (ev, ex_Bonobo_ConfigDatabase_NotFound);
+
 	return NULL;
 }
 
@@ -162,6 +170,8 @@ impl_Bonobo_ConfigDatabase_listDirs (PortableServer_Servant  servant,
 	
 	CORBA_exception_init (&nev);
 
+	bonobo_object_ref (BONOBO_OBJECT (cd));
+
 	for (l = cd->priv->db_list; l != NULL; l = l->next) {
 		info = (DataBaseInfo *)l->data;
 
@@ -191,6 +201,8 @@ impl_Bonobo_ConfigDatabase_listDirs (PortableServer_Servant  servant,
 		}
 	}
 
+	bonobo_object_unref (BONOBO_OBJECT (cd));
+
 	CORBA_exception_free (&nev);
 
 	return cur_list;
@@ -217,6 +229,8 @@ impl_Bonobo_ConfigDatabase_listKeys (PortableServer_Servant  servant,
 
 	CORBA_exception_init (&nev);
 
+	bonobo_object_ref (BONOBO_OBJECT (cd));
+
 	for (l = cd->priv->db_list; l != NULL; l = l->next) {
 		info = (DataBaseInfo *)l->data;
 
@@ -246,6 +260,8 @@ impl_Bonobo_ConfigDatabase_listKeys (PortableServer_Servant  servant,
 		}
 	}
 
+	bonobo_object_unref (BONOBO_OBJECT (cd));
+
 	CORBA_exception_free (&nev);
 
 	return cur_list;
@@ -268,6 +284,8 @@ impl_Bonobo_ConfigDatabase_dirExists (PortableServer_Servant  servant,
 
 	CORBA_exception_init (&nev);
 
+	bonobo_object_ref (BONOBO_OBJECT (cd));
+
 	for (l = cd->priv->db_list; l != NULL; l = l->next) {
 		info = (DataBaseInfo *)l->data;
 
@@ -281,6 +299,8 @@ impl_Bonobo_ConfigDatabase_dirExists (PortableServer_Servant  servant,
 			return TRUE;
 	}
 
+	bonobo_object_unref (BONOBO_OBJECT (cd));
+
 	CORBA_exception_free (&nev);
 
 	return CORBA_FALSE;
@@ -319,15 +339,20 @@ impl_Bonobo_ConfigDatabase_addDatabase (PortableServer_Servant servant,
 	DataBaseInfo *info;
 	GList *l;
 
+	g_return_if_fail (cd->priv != NULL);
+
 	/* we cant add ourselves */
 	if (CORBA_Object_is_equivalent (BONOBO_OBJREF (cd), ddb, NULL))
 		return;
 
-	/* check if the database was already added */
-	for (l = cd->priv->db_list; l != NULL; l = l->next) {
-		info = (DataBaseInfo *)l->data;
-		if (CORBA_Object_is_equivalent (info->db, ddb, NULL))
+	bonobo_object_ref (BONOBO_OBJECT (cd));
+
+	for (l = cd->priv->db_list; l; l = l->next) {
+		info = l->data;
+		if (CORBA_Object_is_equivalent (info->db, ddb, NULL)) {
+			bonobo_object_unref (BONOBO_OBJECT (cd));
 			return;
+		}
 	}
 
 	info = g_new0 (DataBaseInfo , 1);
@@ -342,6 +367,8 @@ impl_Bonobo_ConfigDatabase_addDatabase (PortableServer_Servant servant,
 	info->path = g_strdup (path);
 
 	cd->priv->db_list = g_list_append (cd->priv->db_list, info);
+
+	bonobo_object_unref (BONOBO_OBJECT (cd));
 }
 
 static void 
@@ -367,25 +394,35 @@ static void
 bonobo_config_database_destroy (GtkObject *object)
 {
 	BonoboConfigDatabase *cd = BONOBO_CONFIG_DATABASE (object);
-	DataBaseInfo *info;
+	GList *db_list;
 	GList *l;
 
-	for (l = cd->priv->db_list; l != NULL; l = l->next) {
-		info = (DataBaseInfo *)l->data;
+	db_list = cd->priv->db_list;
+	cd->priv->db_list = NULL;
+
+	for (l = db_list; l; l = l->next) {
+		DataBaseInfo *info = l->data;
 
 		bonobo_object_release_unref (info->db, NULL);
 
 		g_free (info->path);
-
 		g_free (info);
 	}
 	
-	if (cd->priv->db_list)
-		g_list_free (cd->priv->db_list);
+	g_list_free (db_list);
+
+	parent_class->destroy (object);
+}
+
+static void
+bonobo_config_database_finalize (GtkObject *object)
+{
+	BonoboConfigDatabase *cd = (BonoboConfigDatabase *) object;
 
 	g_free (cd->priv);
-	
-	parent_class->destroy (object);
+	cd->priv = NULL;
+
+	parent_class->finalize (object);
 }
 
 static void
@@ -397,6 +434,7 @@ bonobo_config_database_class_init (BonoboConfigDatabaseClass *class)
 	parent_class = gtk_type_class (PARENT_TYPE);
 
 	object_class->destroy = bonobo_config_database_destroy;
+	object_class->finalize = bonobo_config_database_finalize;
 
 	epv = &class->epv;
 
@@ -732,8 +770,11 @@ bonobo_config_get_value  (Bonobo_ConfigDatabase  db,
 
 	if (BONOBO_EX (my_ev)) {
 		if (!opt_ev) {
-			g_warning ("Cannot get value: %s\n", 
-				   bonobo_exception_get_text (my_ev));
+			char *err;
+
+			err = bonobo_exception_get_text (my_ev);
+			g_warning ("Cannot get value: %s\n", err);
+			g_free (err);
 			
 			CORBA_exception_free (&ev);
 		}
