@@ -82,27 +82,32 @@ demo3_get_fn (BonoboConfigControl *control,
 	      CORBA_Environment   *ev)
 {
 	GtkWidget *w, *table;
+	BonoboPEditor *ed;
 
 	table = gtk_table_new (3, 3, FALSE);
 
 	w = gtk_label_new ("A normal integer value");
 	gtk_table_attach_defaults (GTK_TABLE (table), w, 0, 1, 0, 1);
 
-	w = bonobo_property_editor_new (pb, "test-long2", TC_long, NULL);
+	ed = BONOBO_PEDITOR (bonobo_peditor_new (pb, "test-long2", TC_long, 
+						 NULL));
+	w = bonobo_peditor_get_widget (ed);
 	gtk_table_attach_defaults (GTK_TABLE (table), w, 1, 2, 0, 1);
 
 	w = gtk_label_new ("Enumeration Types");
 	gtk_table_attach_defaults (GTK_TABLE (table), w, 0, 1, 1, 2);
 
-	w = bonobo_property_editor_new (pb, "test-enum", TC_Bonobo_StorageType,
-					NULL);
+	ed = BONOBO_PEDITOR (bonobo_peditor_new (pb, "test-enum", 
+						 TC_Bonobo_StorageType, NULL));
+	w = bonobo_peditor_get_widget (ed);
 	gtk_table_attach_defaults (GTK_TABLE (table), w, 1, 2, 1, 2);
 
 	w = gtk_label_new ("Filename");
 	gtk_table_attach_defaults (GTK_TABLE (table), w, 0, 1, 2, 3);
 
-	w = bonobo_property_editor_new (pb, "test-filename", 
-					TC_Bonobo_Config_FileName, NULL);
+	ed = BONOBO_PEDITOR (bonobo_peditor_new (pb, "test-filename", 
+	        TC_Bonobo_Config_FileName, NULL));
+	w = bonobo_peditor_get_widget (ed);
 	gtk_table_attach_defaults (GTK_TABLE (table), w, 1, 2, 2, 3);
 
 	gtk_widget_show_all (table);
@@ -210,6 +215,7 @@ static void
 demo5 ()
 {
 	GtkWidget *w, *d, *l, *v0, *v, *h, *pf, *f;
+	BonoboPEditor *ed;
 	CORBA_Environment ev;
 	Bonobo_PropertyBag bag;
 	char *titles[] = { "Tiled", "Centered", "Scaled (keep aspect)", 
@@ -243,20 +249,24 @@ demo5 ()
 	v = gtk_vbox_new (FALSE, 5);
 	gtk_container_add (GTK_CONTAINER (f), v);
 
-	w = bonobo_property_editor_new (bag, "test-filename", 
-					TC_Bonobo_Config_FileName, NULL);
-	bonobo_property_editor_set_guard (w, bag, "use-gnome");
+	ed = BONOBO_PEDITOR (bonobo_peditor_new (bag, "test-filename", 
+	        TC_Bonobo_Config_FileName, NULL));
+	w = bonobo_peditor_get_widget (ed);
+	bonobo_peditor_set_guard (w, bag, "use-gnome");
 	gtk_box_pack_start_defaults (GTK_BOX (v), w);
 
-	w = bonobo_property_editor_option_new (1, titles);
-	bonobo_property_editor_set_property (BONOBO_PROPERTY_EDITOR (w),
-					     bag, "test-radio", TC_ulong, NULL);
-	bonobo_property_editor_set_guard (w, bag, "use-gnome");
+	ed = BONOBO_PEDITOR (bonobo_peditor_option_new (1, titles));
+	w = bonobo_peditor_get_widget (ed);
+	bonobo_peditor_set_property (BONOBO_PEDITOR (ed),
+				     bag, "test-radio", TC_ulong, NULL);
+	bonobo_peditor_set_guard (w, bag, "use-gnome");
 	gtk_box_pack_start_defaults (GTK_BOX (v), w);
 	
-	/* we create a PropertyEditor for long integers */
-	w = bonobo_property_editor_new (bag, "test-long2", TC_long, NULL);
-	bonobo_property_editor_set_guard (w, bag, "use-gnome");
+	/* we create a PEditor for long integers */
+	ed = BONOBO_PEDITOR (bonobo_peditor_new (bag, "test-long2", TC_long, 
+						 NULL));
+	w = bonobo_peditor_get_widget (ed);
+	bonobo_peditor_set_guard (w, bag, "use-gnome");
 	gtk_box_pack_start_defaults (GTK_BOX (v), w);
 
 	/* Color */
@@ -268,8 +278,10 @@ demo5 ()
 	gtk_container_add (GTK_CONTAINER (f), v);
 
 	/* an editor for the same property, just to show that it works */
-	w = bonobo_property_editor_new (bag, "test-long2", TC_long, NULL);
-	bonobo_property_editor_set_guard (w, bag, "use-gnome");
+	ed = BONOBO_PEDITOR (bonobo_peditor_new (bag, "test-long2", TC_long, 
+						 NULL));
+	w = bonobo_peditor_get_widget (ed);
+	bonobo_peditor_set_guard (w, bag, "use-gnome");
 	gtk_box_pack_start_defaults (GTK_BOX (v), w);
 
 	h = gtk_hbox_new (FALSE, 5);
@@ -278,11 +290,12 @@ demo5 ()
 	l = gtk_label_new ("Primary Color");
 	gtk_box_pack_start_defaults (GTK_BOX (h), l);
 
-	/* add a PropertyEditor for the primary colors */
+	/* add a PEditor for the primary colors */
 
-	w = bonobo_property_editor_new (bag, "primary-color", 
-					TC_Bonobo_Config_Color, NULL);
-	bonobo_property_editor_set_guard (w, bag, "use-gnome");
+	ed = BONOBO_PEDITOR (bonobo_peditor_new (bag, "primary-color", 
+	        TC_Bonobo_Config_Color, NULL));
+	w = bonobo_peditor_get_widget (ed);
+	bonobo_peditor_set_guard (w, bag, "use-gnome");
 
 	gtk_box_pack_start (GTK_BOX (h), w, 0, 0, 0);
 
@@ -293,11 +306,12 @@ demo5 ()
 	gtk_box_pack_start_defaults (GTK_BOX (h), l);
 
 
-	/* add a PropertyEditor for the secondary colors */
+	/* add a PEditor for the secondary colors */
 
-	w = bonobo_property_editor_new (bag, "secondary-color", 
-					TC_Bonobo_Config_Color, NULL);
-	bonobo_property_editor_set_guard (w, bag, "use-gnome");
+	ed = BONOBO_PEDITOR (bonobo_peditor_new (bag, "secondary-color", 
+	        TC_Bonobo_Config_Color, NULL));
+	w = bonobo_peditor_get_widget (ed);
+	bonobo_peditor_set_guard (w, bag, "use-gnome");
 	gtk_box_pack_start (GTK_BOX (h), w, 0, 0, 0);
 
 
@@ -308,7 +322,9 @@ demo5 ()
 	gtk_box_pack_start_defaults (GTK_BOX (h), l);
 
 	/* a guard property */
-	w = bonobo_property_editor_new (bag, "use-gnome", TC_boolean, NULL);
+	ed = BONOBO_PEDITOR (bonobo_peditor_new (bag, "use-gnome", TC_boolean,
+						 NULL));
+	w = bonobo_peditor_get_widget (ed);
 	gtk_box_pack_start (GTK_BOX (h), w, 0, 0, 0);
 
 	bonobo_event_source_client_add_listener (bag, demo5_cb, NULL, NULL, 
