@@ -139,20 +139,68 @@ real_get_value (BonoboConfigDatabase *db,
 {
 	BonoboConfigGConfDB *gconfdb = BONOBO_CONFIG_GCONFDB (db);
 	GConfEntry          *ge;
+	GConfSchema         *s;
 	CORBA_any           *value = NULL;
+	char                *rk;
+	int                  t = 0;
 
-	/* disabled locale because it is not supported by GConf */
-	ge = gconf_client_get_entry (gconfdb->client, key, 
-				     /* locale */ NULL, 0, NULL);
+	if (!strncmp (key, "/doc/short/", 11) && key [11]) {
+		t = 1;
+		rk = g_strdup (&key [10]);
+	} else if (!strncmp (key, "/doc/long/", 10) && key [10]) {
+		t = 2;
+		rk = g_strdup (&key [9]);
+	} else 
+		rk = g_strdup (key);
 	
-	if (ge) { 
-		value = gconf_to_corba_any (ge->value);
-		gconf_entry_free (ge);
+	/* disabled locale because it is not supported by GConf */
+	ge = gconf_client_get_entry (gconfdb->client, rk, /* locale */ NULL, 
+				     TRUE, NULL);
+	
+	g_free (rk);
+
+	if (!t) {
+		if (ge) { 
+			value = gconf_to_corba_any (ge->value);
+			gconf_entry_free (ge);
+		}
+
+		if (!value)
+			bonobo_exception_set (ev, 
+			        ex_Bonobo_ConfigDatabase_NotFound);
+		
+		return value;
 	}
 
+	/* this does not work at all, because GConf does not return 
+	 * schema_name (always get NULL) */  
+
+	if (!ge || !ge->schema_name) {
+		bonobo_exception_set (ev, ex_Bonobo_ConfigDatabase_NotFound);
+		return NULL;
+	}
+
+	rk = g_strdup (ge->schema_name);
+	gconf_entry_free (ge);
+
+	if ((s = gconf_client_get_schema (gconfdb->client, rk, NULL))) {
+
+		if (t == 1 && s->short_desc) {
+			value = bonobo_arg_new (TC_string);
+			BONOBO_ARG_SET_STRING (value, s->short_desc);
+		} else if (t == 2 && s->long_desc) {
+			value = bonobo_arg_new (TC_string);
+			BONOBO_ARG_SET_STRING (value, s->long_desc);
+		}
+		
+		gconf_schema_free (s);
+	}
+
+	g_free (rk);
+
 	if (!value)
 		bonobo_exception_set (ev, ex_Bonobo_ConfigDatabase_NotFound);
-		
+
 	return value;
 }
 
@@ -176,6 +224,12 @@ real_set_value (BonoboConfigDatabase *db,
 	GConfValue          *gv;
 	GError              *err = NULL;
 
+	if (!strncmp (key, "/doc/short/", 11) ||
+	    !strncmp (key, "/doc/long/", 10)) {
+		bonobo_exception_set (ev, ex_Bonobo_ConfigDatabase_NotFound);
+		return;
+	}
+
 	if (!(gv = corba_any_to_gconf (value))) 
 		return;
 
@@ -209,9 +263,10 @@ real_list_dirs (BonoboConfigDatabase *db,
 	}
 
 	key_list = Bonobo_KeyList__alloc ();
-	key_list->_length = len = g_slist_length (gcl);
+	key_list->_length = 0;
+	len = g_slist_length (gcl);
 
-	if (!key_list->_length)
+	if (!len)
 		return key_list;
 	
 	key_list->_buffer = CORBA_sequence_CORBA_string_allocbuf (len);
@@ -250,9 +305,10 @@ real_list_keys (BonoboConfigDatabase *db,
 	}
 
 	key_list = Bonobo_KeyList__alloc ();
-	key_list->_length = len = g_slist_length (gcl);
+	key_list->_length = 0;
+	len = g_slist_length (gcl);
 
-	if (!key_list->_length)
+	if (!len)
 		return key_list;
 	
 	key_list->_buffer = CORBA_sequence_CORBA_string_allocbuf (len);
@@ -261,7 +317,7 @@ real_list_keys (BonoboConfigDatabase *db,
 	for (l = gcl; l != NULL; l = l->next) {
 		ge = (GConfEntry *)l->data;
 		key_list->_buffer [key_list->_length] = 
-			CORBA_string_dup (ge->key);
+			CORBA_string_dup (g_basename (ge->key));
 		gconf_entry_free (ge);
 		key_list->_length++;
 	}
