@@ -206,34 +206,316 @@ real_get_value (BonoboConfigDatabase *db,
 	return value;
 }
 
+
+static void xmlNodeDump (xmlBufferPtr buf, xmlDocPtr doc, xmlNodePtr cur, int level, int format);
+
+
+static void
+xmlAttrDump (xmlBufferPtr buf, xmlDocPtr doc, xmlAttrPtr cur)
+{
+	xmlChar *value;
+	
+	if (cur == NULL) {
+#ifdef DEBUG_TREE
+		fprintf(stderr, "xmlAttrDump : property == NULL\n");
+#endif
+		return;
+	}
+	
+	xmlBufferWriteChar (buf, " ");
+	if ((cur->ns != NULL) && (cur->ns->prefix != NULL)) {
+		xmlBufferWriteCHAR (buf, cur->ns->prefix);
+		xmlBufferWriteChar (buf, ":");
+	}
+	
+	xmlBufferWriteCHAR (buf, cur->name);
+	value = xmlNodeListGetString (doc, cur->val, 0);
+	if (value) {
+		xmlBufferWriteChar (buf, "=");
+		xmlBufferWriteQuotedString (buf, value);
+		xmlFree (value);
+	} else  {
+		xmlBufferWriteChar (buf, "=\"\"");
+	}
+}
+
+static void
+xmlAttrListDump (xmlBufferPtr buf, xmlDocPtr doc, xmlAttrPtr cur)
+{
+	if (cur == NULL) {
+#ifdef DEBUG_TREE
+		fprintf(stderr, "xmlAttrListDump : property == NULL\n");
+#endif
+		return;
+	}
+	
+	while (cur != NULL) {
+		xmlAttrDump (buf, doc, cur);
+		cur = cur->next;
+	}
+}
+
+static void
+xmlNodeListDump (xmlBufferPtr buf, xmlDocPtr doc, xmlNodePtr cur, int level, int format)
+{
+	int i;
+	
+	if (cur == NULL) {
+#ifdef DEBUG_TREE
+		fprintf(stderr, "xmlNodeListDump : node == NULL\n");
+#endif
+		return;
+	}
+	
+	while (cur != NULL) {
+		if ((format) && (xmlIndentTreeOutput) &&
+		    (cur->type == XML_ELEMENT_NODE))
+			for (i = 0; i < level; i++)
+				xmlBufferWriteChar (buf, "  ");
+		xmlNodeDump (buf, doc, cur, level, format);
+		if (format) {
+			xmlBufferWriteChar (buf, "\n");
+		}
+		cur = cur->next;
+	}
+}
+
+static void
+xmlNodeDump (xmlBufferPtr buf, xmlDocPtr doc, xmlNodePtr cur, int level, int format)
+{
+	int i;
+	xmlNodePtr tmp;
+	
+	if (cur == NULL) {
+#ifdef DEBUG_TREE
+		fprintf(stderr, "xmlNodeDump : node == NULL\n");
+#endif
+		return;
+	}
+	
+	if (cur->type == XML_TEXT_NODE) {
+		if (cur->content != NULL) {
+			xmlChar *buffer;
+			
+#ifndef XML_USE_BUFFER_CONTENT
+			buffer = xmlEncodeEntitiesReentrant (doc, cur->content);
+#else
+			buffer = xmlEncodeEntitiesReentrant (doc, xmlBufferContent (cur->content));
+#endif
+			if (buffer != NULL) {
+				xmlBufferWriteCHAR (buf, buffer);
+				xmlFree (buffer);
+			}
+		}
+		return;
+	}
+	
+	if (cur->type == XML_PI_NODE) {
+		if (cur->content != NULL) {
+			xmlBufferWriteChar (buf, "<?");
+			xmlBufferWriteCHAR (buf, cur->name);
+			if (cur->content != NULL) {
+				xmlBufferWriteChar (buf, " ");
+#ifndef XML_USE_BUFFER_CONTENT
+				xmlBufferWriteCHAR (buf, cur->content);
+#else
+				xmlBufferWriteCHAR (buf, xmlBufferContent (cur->content));
+#endif
+			}
+			xmlBufferWriteChar (buf, "?>");
+		}
+		return;
+	}
+	
+	if (cur->type == XML_COMMENT_NODE) {
+		if (cur->content != NULL) {
+			xmlBufferWriteChar (buf, "<!--");
+#ifndef XML_USE_BUFFER_CONTENT
+			xmlBufferWriteCHAR (buf, cur->content);
+#else
+			xmlBufferWriteCHAR (buf, xmlBufferContent (cur->content));
+#endif
+			xmlBufferWriteChar (buf, "-->");
+		}
+		return;
+	}
+	
+	if (cur->type == XML_ENTITY_REF_NODE) {
+		xmlBufferWriteChar (buf, "&");
+		xmlBufferWriteCHAR (buf, cur->name);
+		xmlBufferWriteChar (buf, ";");
+		return;
+	}
+	
+	if (cur->type == XML_CDATA_SECTION_NODE) {
+		xmlBufferWriteChar (buf, "<![CDATA[");
+		if (cur->content != NULL)
+#ifndef XML_USE_BUFFER_CONTENT
+			xmlBufferWriteCHAR (buf, cur->content);
+#else
+		xmlBufferWriteCHAR (buf, xmlBufferContent(cur->content));
+#endif
+		xmlBufferWriteChar (buf, "]]>");
+		return;
+	}
+	
+	if (format == 1) {
+		tmp = cur->childs;
+		while (tmp != NULL) {
+			if ((tmp->type == XML_TEXT_NODE) || 
+			    (tmp->type == XML_ENTITY_REF_NODE)) {
+				format = 0;
+				break;
+			}
+			tmp = tmp->next;
+		}
+	}
+	
+	xmlBufferWriteChar (buf, "<");
+	if ((cur->ns != NULL) && (cur->ns->prefix != NULL)) {
+		xmlBufferWriteCHAR (buf, cur->ns->prefix);
+		xmlBufferWriteChar (buf, ":");
+	}
+	
+	xmlBufferWriteCHAR (buf, cur->name);
+	
+	if (cur->properties != NULL)
+		xmlAttrListDump (buf, doc, cur->properties);
+	
+	if ((cur->content == NULL) && (cur->childs == NULL) &&
+	    (!xmlSaveNoEmptyTags)) {
+		xmlBufferWriteChar (buf, "/>");
+		return;
+	}
+	
+	xmlBufferWriteChar (buf, ">");
+	if (cur->content != NULL) {
+		xmlChar *buffer;
+		
+#ifndef XML_USE_BUFFER_CONTENT
+		buffer = xmlEncodeEntitiesReentrant (doc, cur->content);
+#else
+		buffer = xmlEncodeEntitiesReentrant (doc, xmlBufferContent (cur->content));
+#endif
+		if (buffer != NULL) {
+			xmlBufferWriteCHAR (buf, buffer);
+			xmlFree (buffer);
+		}
+	}
+	
+	if (cur->childs != NULL) {
+		if (format)
+			xmlBufferWriteChar (buf, "\n");
+		
+		xmlNodeListDump (buf, doc, cur->childs, (level >= 0 ? level + 1 : -1), format);
+		if ((xmlIndentTreeOutput) && (format))
+			for (i = 0; i < level; i++)
+				xmlBufferWriteChar (buf, "  ");
+	}
+	
+	xmlBufferWriteChar (buf, "</");
+	if ((cur->ns != NULL) && (cur->ns->prefix != NULL)) {
+		xmlBufferWriteCHAR (buf, cur->ns->prefix);
+		xmlBufferWriteChar (buf, ":");
+	}
+	
+	xmlBufferWriteCHAR (buf, cur->name);
+	xmlBufferWriteChar (buf, ">");
+}
+
+static void
+xmlDocContentDump (xmlBufferPtr buf, xmlDocPtr cur)
+{
+	xmlBufferWriteChar (buf, "<?xml version=");
+	
+	if (cur->version != NULL)
+		xmlBufferWriteQuotedString (buf, cur->version);
+	else
+		xmlBufferWriteChar (buf, "\"1.0\"");
+	
+	if ((cur->encoding != NULL) &&
+	    (strcasecmp (cur->encoding, "UTF-8") != 0)) {
+		xmlBufferWriteChar (buf, " encoding=");
+		xmlBufferWriteQuotedString (buf, cur->encoding);
+	}
+	
+	switch (cur->standalone) {
+        case 1:
+		xmlBufferWriteChar (buf, " standalone=\"yes\"");
+		break;
+	}
+	
+	xmlBufferWriteChar (buf, "?>\n");
+	if (cur->root != NULL) {
+		xmlNodePtr child = cur->root;
+		
+		while (child != NULL) {
+			xmlNodeDump (buf, cur, child, 0, 1);
+			xmlBufferWriteChar (buf, "\n");
+			child = child->next;
+		}
+	}
+}
+
 static void
 real_sync (BonoboConfigDatabase *db, 
 	   CORBA_Environment *ev)
 {
 	BonoboConfigXMLDB *xmldb = BONOBO_CONFIG_XMLDB (db);
+	size_t n, written = 0;
+	xmlBufferPtr buf;
 	char *tmp_name;
+	ssize_t w;
+	int fd;
 	
 	if (!db->writeable)
 		return;
 	
-	tmp_name = g_strdup_printf ("%s.tmp.%d\n", xmldb->filename, getpid ());
-	
-	/* FIXME: if we used xmlBufferDump we could check errno, but
-           we can't do that easily since this xml doc might want to be
-           compressed? I guess we could always just do the compression
-           ourselves? */
-	
-	if (xmlSaveFile (tmp_name, xmldb->doc) < 0) {
-		/* Sigh. There's no way to find out why xmlSaveFile failed.
-		   Ideally we'd set db->writeable to FALSE here if it was a
-		   permissions problem, but since we have no way of finding
-		   out why it failed we can't ever set the db to read-only
-		   here. */
+ retry_open:
+	tmp_name = g_strdup_printf ("%s.tmp.%d.XXXXXX", xmldb->filename, getpid ());
+	if (!mktemp (tmp_name)) {
 		g_free (tmp_name);
 		return;
 	}
 	
-	/* chmod (tmp_name, 0664); */
+	fd = open (tmp_name, O_WRONLY | O_CREAT | O_EXCL, 0600);
+	if (fd == -1) {
+		g_free (tmp_name);
+		if (errno == EEXIST)
+			goto retry_open;
+		
+		return;
+	}
+	
+	if (!(buf = xmlBufferCreate ())) {
+		close (fd);
+		unlink (tmp_name);
+		g_free (tmp_name);
+		return;
+	}
+	
+	xmlDocContentDump (buf, xmldb->doc);
+	
+	n = buf->use;
+	do {
+		do {
+			w = write (fd, buf->content + written, n - written);
+		} while (w == -1 && errno == EINTR);
+		
+		if (w > 0)
+			written += w;
+	} while (w != -1 && written < n);
+	
+	xmlBufferFree (buf);
+	
+	if (written < n || fsync (fd) == -1) {
+		close (fd);
+		unlink (tmp_name);
+		g_free (tmp_name);
+		return;
+	}
+	
+	close (fd);
 	
 	if (rename (tmp_name, xmldb->filename) < 0) {
 		/* if we don't have permissions then we can assume the db is read-only */
@@ -241,12 +523,9 @@ real_sync (BonoboConfigDatabase *db,
 			db->writeable = FALSE;
 		
 		unlink (tmp_name);
-		g_free (tmp_name);
-		return;
 	}
 	
 	g_free (tmp_name);
-	return;
 }
 
 static gint
